<?php

use LECO\Client_Portal\Login;
use LECO\Client_Portal\Notifications;
use LECO\Client_Portal\User;
use LECO\Client_Portal\Integrations;
use LECO\Client_Portal\Templates\Integrations as Template_Integrations;
use LECO\Client_Portal\REST_API\Endpoints\Controller as REST_API_Controller;
use LECO\Client_Portal\Admin\List_View as Admin_List_View;

/**
 * Main LECO_Client_Portal class
 *
 * @since 1.0.0
 */
class LECO_Client_Portal {

	/**
	 * @since 1.0.0
	 * @var LECO_Client_Portal $instance The one true LECO_Client_Portal
	 */
	private static $instance;

	/**
	 * Client Portal HTML Session Object.
	 *
	 * This holds everything stored in the session.
	 *
	 * @since 4.6
	 * @var object|LECO_CP_Session
	 *
	 */
	public $session;

	/**
	 * Client Portal Emails Object.
	 *
	 * @since 4.9
	 * @var object|LECO_CP_Emails
	 */
	public $emails;

	/**
	 * Client Portal User Object.
	 *
	 * @since 4.11
	 * @var User
	 */
	public $user;

	/**
	 * WooCommerce's integration object.
	 *
	 * @since 4.14
	 * @var Integrations\WooCommerce
	 */
	public $wc;

	/**
	 * Notifications object.
	 *
	 * @since 4.15
	 *
	 * @var Notifications
	 */
	private $notifications;

	/**
	 * REST API Controller object.
	 *
	 * @since 5.0
	 *
	 * @var REST_API_Controller
	 */
	private $rest_api_controller;

	/**
	 * Admin services object.
	 *
	 * @var array
	 */
	private $admin;

	/**
	 * Get active instance
	 *
	 * @access public
	 * @since 1.0.0
	 * @return object self::$instance The one true LECO_Client_Portal
	 */
	public static function instance() {
		if ( ! self::$instance ) {
			self::$instance = new LECO_Client_Portal();
			self::$instance->setup_constants();

			// Fix PHP warning thrown if call self::$instance->load_textdomain() directly here.
			add_action( 'plugins_loaded', array( self::$instance, 'load_textdomain' ) );

			self::$instance->includes();
			self::$instance->session             = new LECO_CP_Session();
			self::$instance->user                = User::instance();
			self::$instance->notifications       = Notifications::instance();
			self::$instance->rest_api_controller = new REST_API_Controller();

			if ( is_admin() ) {
				self::$instance->admin = array(
					'list_view' => Admin_List_View::instance(),
				);
			}

			self::$instance->hooks();

			// Initialize the settings page object.
			leco_cp_settings();
		}

		return self::$instance;
	}

	/**
	 * Setup plugin constants
	 *
	 * @access private
	 * @since 1.0.0
	 * @return void
	 */
	private function setup_constants() {
		// Plugin version.
		define( 'LECO_CLIENT_PORTAL_VER', '5.0.1' );

		// Plugin path.
		define( 'LECO_CLIENT_PORTAL_DIR', plugin_dir_path( __FILE__ ) );

		// Plugin URL.
		define( 'LECO_CLIENT_PORTAL_URL', plugin_dir_url( __FILE__ ) );

		// this is the URL our updater / license checker pings. This should be the URL of the site with EDD installed.
		define( 'LECO_CLIENT_PORTAL_STORE_URL', 'https://client-portal.io' );

		// the name of our product.
		define( 'LECO_CLIENT_PORTAL_ITEM_NAME', 'Client Portal' );
	}


	/**
	 * Include necessary files
	 *
	 * @access private
	 * @since 1.0.0
	 * @return void
	 */
	private function includes() {
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/vendor/autoload.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/rest-api/endpoints/class-controller.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/class-leco-cp-session.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/error-tracking.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/cpt.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/custom-field-types.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/cmb2/types/class-multiple-urls.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/cmb2/types/class-due-date.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/cmb.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/class-project.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/class-user.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/class-login.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/class-notifications.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/class-digest.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/digest/class-settings.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/class-hooks.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/class-notification.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/class-reminder.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/reminder/class-settings.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/class-settings.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/notifications/notification/class-settings.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/class-leco-cp-settings.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/emails/class-leco-cp-emails.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/emails/actions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/actions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/ajax-functions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/download-functions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/content-page/functions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/content-page/actions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/client-uploads.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/gutenberg/patterns.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/gutenberg/blocks.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/gutenberg/meta.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/gutenberg/palette.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/gutenberg/template.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/scripts.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/shortcodes.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/helpers.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/EDD_SL_Plugin_Updater.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'templates/includes/functions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'templates/includes/login-actions.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'templates/includes/login-functions.php';

		if ( is_admin() ) {
			require_once LECO_CLIENT_PORTAL_DIR . 'includes/admin/class-list-view.php';
			require_once LECO_CLIENT_PORTAL_DIR . 'includes/admin/class-permalink-settings.php';
			require_once LECO_CLIENT_PORTAL_DIR . 'includes/admin/import-export.php';
			require_once LECO_CLIENT_PORTAL_DIR . 'includes/admin/upload-functions.php';
		}
	}


	/**
	 * Run action and filter hooks
	 *
	 * @access private
	 * @since 1.0.0
	 * @return void
	 */
	private function hooks() {
		add_filter( 'plugin_action_links_' . plugin_basename( LECO_CP_MAIN_FILE ), array( $this, 'plugin_action_links' ) );
		add_filter( 'show_admin_bar', array( $this, 'show_admin_bar' ) ); // Disable admin bar in CP template.
		add_action( 'admin_init', array( $this, 'plugin_updater' ), 0 );
		add_action( 'admin_init', array( $this, 'install_roles' ) );
		add_action( 'admin_init', array( $this, 'add_custom_menu' ) );

		add_action( 'admin_init', array( $this, 'prevent_admin_access' ) );
		// Use priority 9 so we can pre-set redirect_to parameter for the WP 2FA and Two Factor plugin.
		add_action( 'wp_login', array( $this, 'wp_login' ), 9, 2 );

		add_action( 'admin_init', array( $this, 'remove_actions' ) );

		add_action( 'admin_notices', array( $this, 'admin_notices' ) );

		add_action( 'init', array( $this, 'endpoint' ), 11 );
		add_action( 'template_redirect', array( $this, 'load_template_functions' ) );
		add_action( 'parse_request', array( $this, 'download_private_file' ), 0 );

		// Fix compatibility issue with Permalink Manager plugin.
		add_filter( 'request', array( $this, 'fix_permalink_manager' ) );
		// Skip the force redirect from the Wishlist member plugin.
		add_filter(
			'wishlistmember_login_redirect_override',
			array(
				$this,
				'wishlistmember_login_redirect_override',
			)
		);

		// Init emails object in `cmb2_init` because CMB was included very late in the `init` action.
		add_action( 'cmb2_init', array( $this, 'init_emails' ) );

		// Run hooks from objects.
		$this->user->hooks();
		$this->notifications->hooks();

		if ( is_admin() ) {
			/* @var $list_view Admin_List_View */
			$list_view = $this->admin['list_view'];
			$list_view->hooks();
		}

		// Add integrations with other plugins.
		add_action( 'plugins_loaded', array( $this, 'add_integrations' ), 11 );

		// Send test notifications.
		foreach ( $this->notifications->get_events() as $event ) {
			add_action( "leco_cp_test-$event", 'leco_cp_send_test_notification' );
		}
		add_action( 'leco_cp_test-digest', 'leco_cp_send_test_notification' );
		add_action( 'leco_cp_test-reminder', 'leco_cp_send_test_notification' );

		// Fix Bluehost login redirection issue.
		if ( class_exists( 'Bluehost\LoginRedirect' ) ) {
			remove_action( 'login_form_defaults', array( 'Bluehost\LoginRedirect', 'filter_login_form_defaults' ) );
		}

		// Fix compatibility issue with WP User Frontend.
		add_action( 'pre_get_posts', array( $this, 'action_pre_get_posts' ), 20 );

		$this->register_cmb2_types();
	}

	/**
	 * Internationalization
	 *
	 * @access public
	 * @since 1.0.0
	 * @return void
	 */
	public function load_textdomain() {
		// Set filter for language directory
		$lang_dir = LECO_CLIENT_PORTAL_DIR . '/languages/';
		$lang_dir = apply_filters( 'leco_cp_languages_directory', $lang_dir );

		// Traditional WordPress plugin locale filter
		$locale = apply_filters( 'plugin_locale', get_locale(), 'leco-cp' );
		$mofile = sprintf( '%1$s-%2$s.mo', 'leco-cp', $locale );

		// Setup paths to current locale file
		$mofile_local  = $lang_dir . $mofile;
		$mofile_global = WP_LANG_DIR . '/leco-cp/' . $mofile;

		// Look in wp-content/languages/leco-client-portal/
		$mofile_global1 = WP_LANG_DIR . '/leco-client-portal/' . $mofile;

		// Look in wp-content/languages/plugins/leco-client-portal/
		$mofile_global2 = WP_LANG_DIR . '/plugins/leco-client-portal/' . $mofile;

		if ( file_exists( $mofile_global ) ) {
			// Look in global /wp-content/languages/leco-cp/ folder
			load_textdomain( 'leco-cp', $mofile_global );
		} elseif ( file_exists( $mofile_global1 ) ) {
			// Look in wp-content/languages/leco-client-portal/ folder
			load_textdomain( 'leco-cp', $mofile_global1 );
		} elseif ( file_exists( $mofile_global2 ) ) {
			// Look in wp-content/languages/plugins/leco-client-portal/ folder
			load_textdomain( 'leco-cp', $mofile_global2 );
		} elseif ( file_exists( $mofile_local ) ) {
			// Look in local /wp-content/plugins/leco-cp/languages/ folder
			load_textdomain( 'leco-cp', $mofile_local );
		} else {
			// Load the default language files
			load_plugin_textdomain( 'leco-cp', false, $lang_dir );
		}
	}


	/**
	 * Add plugin action links
	 *
	 * @access public
	 * @since 1.0.0
	 *
	 * @param array $links The existing plugin links array
	 *
	 * @return array The modified links array
	 */
	function plugin_action_links( $links ) {
		$links[] = '<a href="' . esc_url( get_admin_url( null, 'edit.php?post_type=leco_client&page=leco_cp_options' ) ) . '">' . esc_html__( 'Settings', 'leco-cp' ) . '</a>';

		return $links;
	}

	/**
	 * Create login page
	 *
	 * @access public
	 * @since  2.0.0
	 */
	function create_login_page() {
		// Create login page
		$login_page_exists = get_page_by_path( 'client-portal-login' );
		if ( is_null( $login_page_exists ) ) {
			wp_insert_post(
				array(
					'post_title'     => esc_html__( 'Client Portal Login', 'leco-cp' ),
					'post_content'   => '',
					'post_status'    => 'publish',
					'post_author'    => 1, // @todo maybe change to current user id
					'post_type'      => 'page',
					'comment_status' => 'closed',
					'post_name'      => 'client-portal-login',
				)
			);
		}
	}

	/**
	 * Add custom menu as the reference of our menu block.
	 *
	 * @since 5.0
	 *
	 * @return void
	 */
	public function add_custom_menu() {
		$nav_post = get_posts(
			array(
				'post_type' => 'wp_navigation',
				'name'      => 'leco-cp-navigation',
			)
		);

		if ( $nav_post ) {
			return;
		}

		$content = '<!-- wp:home-link {"label":"Home"} /--> <!-- wp:leco-cp/dashboard-link /--> <!-- wp:leco-cp/all-projects-link /--> <!-- wp:leco-cp/logout-link /-->';

		$postarr = array(
			'post_content' => $content,
			'post_title'   => __( 'Client Portal Menu', 'leco-cp' ),
			'post_name'    => 'leco-cp-navigation',
			'post_type'    => 'wp_navigation',
			'post_status'  => 'publish',
		);

		wp_insert_post(
			$postarr,
			true
		);
	}

	function show_admin_bar( $bool ) {
		global $post;

		if ( empty( $post ) ) {
			global $wp_query;
			if ( $wp_query->is_singular ) {
				$post = get_page_by_path( $wp_query->query['name'], OBJECT, 'leco_template' );
			}
		}

		if ( ! empty( $post ) && ( in_array( $post->post_type, leco_cp_get_post_types() ) || 'client-portal-login' === $post->post_name ) ) {
			return apply_filters( 'leco_cp_show_admin_bar', false );
		}

		return $bool;
	}

	function plugin_updater() {
		// retrieve our license key from the DB
		$license_key = trim( get_option( 'leco_cp_license_key' ) );

		// set up the updater.
		new LECO_Client_Portal_EDD_SL_Plugin_Updater(
			LECO_CLIENT_PORTAL_STORE_URL,
			LECO_CP_MAIN_FILE,
			array(
				'version'   => LECO_CLIENT_PORTAL_VER,                // current version number
				'license'   => $license_key,        // license key (used get_option above to retrieve from DB)
				'item_name' => LECO_CLIENT_PORTAL_ITEM_NAME,    // name of this plugin
				'author'    => 'Laura Elizabeth',  // author of this plugin
			)
		);
	}


	/**
	 * Install user roles on sub-sites of a network
	 *
	 * Roles do not get created when CP is network activation so we need to create them during admin_init
	 *
	 * @since 3.1.9 Added for creating roles on multisite network.
	 * @since 4.3.8 Updated the role name.
	 *
	 * @return void
	 */
	public function install_roles() {

		global $wp_roles;

		if ( ! is_object( $wp_roles ) ) {
			return;
		}

		if ( empty( $wp_roles->roles ) || ! array_key_exists( 'leco_client', $wp_roles->roles ) ) {
			add_role(
				'leco_client',
				esc_html__( 'CP Client', 'leco-cp' ),
				array(
					'read'    => true,
					'level_0' => true,
				)
			);
		} elseif ( array_key_exists( 'leco_client', $wp_roles->roles ) && 'Client' === $wp_roles->roles['leco_client']['name'] ) {
			remove_role( 'leco_client' );
			add_role(
				'leco_client',
				esc_html__( 'CP Client', 'leco-cp' ),
				array(
					'read'    => true,
					'level_0' => true,
				)
			);
		}

	}

	/**
	 * Init the LECO_CP_Emails() object.
	 *
	 * @since 4.9
	 */
	public function init_emails() {
		self::$instance->emails = new LECO_CP_Emails();
	}

	/**
	 * Add endpoint for content pages.
	 *
	 * @since 4.6   Add new endpoint for private files.
	 * @since 4.3.0
	 */
	public function endpoint() {
		add_rewrite_tag( '%leco_content_page%', '([^/]+)' );
		add_rewrite_rule( leco_cp_get_slug() . '/?([^/]*)/module/?([^/]*)', 'index.php?leco_client=$matches[1]&leco_content_page=$matches[2]', 'top' );
		add_rewrite_rule( 'project-template/?([^/]*)/module/?([^/]*)', 'index.php?leco_template=$matches[1]&leco_content_page=$matches[2]', 'top' );

		add_rewrite_tag( '%leco_private_file%', '([^/]+)' );
		add_rewrite_rule(  leco_cp_get_slug() . '/?([^/]*)/file/?([^/]*)', 'index.php?leco_client=$matches[1]&leco_private_file=$matches[2]', 'top' );
		add_rewrite_rule( 'project-template/?([^/]*)/file/?([^/]*)', 'index.php?leco_template=$matches[1]&leco_private_file=$matches[2]', 'top' );
	}

	/**
	 * Fix compatibility issue with Permalink Manager plugin.
	 *
	 * @param array $query_vars The array of requested query variables.
	 *
	 * @return mixed
	 */
	public function fix_permalink_manager( $query_vars ) {
		if ( class_exists( 'Permalink_Manager_Class' ) ) {
			if ( isset( $query_vars['leco_content_page'] ) && isset( $query_vars['post_type'] ) && ( 'leco_client' === $query_vars['post_type'] || 'leco_template' === $query_vars['post_type'] ) ) {
				$query_vars['do_not_redirect'] = 1;
			}
		}

		return $query_vars;
	}

	/**
	 * Prevent any user who cannot 'edit_posts' (subscribers, customers etc) from accessing admin.
	 * Borrowed from WooCommerce.
	 */
	public function prevent_admin_access() {
		// Do not prevent access if the WP 2FA or Two Factor plugin is active.
		if ( class_exists( '\WP2FA\Authenticator\Login' ) || class_exists( '\Two_Factor_Core' ) ) {
			return;
		}

		$prevent_access = false;

		if ( 'yes' === get_option( 'leco_cp_lock_down_admin', 'yes' ) && ! defined( 'DOING_AJAX' ) && basename( $_SERVER['SCRIPT_FILENAME'] ) !== 'admin-post.php' ) {
			if ( current_user_can( 'leco_client' ) && ! current_user_can( 'edit_posts' ) ) {
				$prevent_access = true;
			}
		}

		if ( apply_filters( 'leco_cp_prevent_admin_access', $prevent_access ) ) {
			wp_safe_redirect( leco_cp_login_url() );
			exit;
		}
	}

	/**
	 * Filter the action of wp_login for the WP 2FA and Two Factor plugin.
	 *
	 * Set the redirect_to parameter to the CP login page, so users can be redirected to their portal after the 2FA validation.
	 *
	 * @since 4.16.6.1
	 *
	 * @param string  $user_login Username.
	 * @param WP_User $user       \WP_User object of the logged-in user.
	 *
	 * @return void
	 */
	public function wp_login( $user_login, $user ) {

		if ( ! class_exists( '\WP2FA\Authenticator\Login' ) && ! class_exists( '\Two_Factor_Core' ) ) {
			return;
		}

		if ( ! user_can( $user, 'leco_client' ) ) {
			return;
		}

		$_REQUEST['redirect_to'] = Login::login_url();

	}

	/**
	 * Remove actions, mostly from other plugins to solve compatibility issues.
	 */
	public function remove_actions() {
		// Fix for Scripts n Styles.
		if ( class_exists( 'SnS_Admin_Meta_Box' ) ) {
			if ( isset( $_GET['page'] ) && 'leco_cp_options' === $_GET['page'] ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				remove_all_actions( 'current_screen' );
			}
		}
	}

	/**
	 * Trigger private file downloading
	 *
	 * @since 4.6
	 */
	public function download_private_file() {
		leco_cp_download_private_file();
	}

	/**
	 * Admin notices in Client Portal.
	 *
	 * @since 4.14 Remove the leco_cp_tailwind_admin_notices().
	 * @since 4.7 Change method name and added leco_cp_tailwind_admin_notices().
	 * @since 4.6
	 */
	public function admin_notices() {
		global $post_type, $typenow;

		$ptype = $post_type;
		if ( $ptype === null && $typenow !== null ) {
			$ptype = $typenow;
		}

		if ( ! isset( $ptype ) || ! in_array(
				$ptype,
				array(
					'leco_client',
					'leco_template',
					'leco_content_page',
				),
				true
			) ) {
			return;
		}

		leco_cp_admin_notices_for_nginx();

		// Added in 4.8.
		leco_cp_admin_notices_for_nginx_client_uploads();
	}

	/**
	 * Load template own functions.
	 *
	 * @since 4.7
	 */
	public function load_template_functions() {
		$leco_cp_template = leco_cp_get_template();
		$leco_cp_template_file = LECO_CLIENT_PORTAL_DIR . "templates/$leco_cp_template/functions.php";
		if ( file_exists( $leco_cp_template_file ) ) {
			require_once $leco_cp_template_file;
		}
	}

	/**
	 * Skip the force redirect from the Wishlist member plugin.
	 *
	 * @since 4.8.6
	 *
	 * @return bool
	 */
	public function wishlistmember_login_redirect_override() {
		$redirect_to = ( ! empty( $_REQUEST['redirect_to'] ) ) ? esc_url( $_REQUEST['redirect_to'] ) : ''; // phpcs:ignore

		if ( Login::is_login_page( $redirect_to ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Dequeue styles and scripts that caused compatibility issue.
	 *
	 * @since 4.9.5
	 */
	public function dequeue_scripts() {
		if ( class_exists( 'Everlightbox' ) ) {
			wp_dequeue_style( 'everlightbox' );
			wp_dequeue_script( 'everlightbox' );
		}
	}

	/**
	 * Fix compatibility issue with WP User Frontend.
	 *
	 * @since 4.13.3
	 *
	 * @return void
	 */
	public function action_pre_get_posts() {

		if ( function_exists( 'wpuf' ) && defined( 'WPUF_VERSION' ) && version_compare( WPUF_VERSION, '3.5.26' ) ) {
			// The following filter was adding in 3.5.27.
			remove_filter( 'user_has_cap', [ wpuf()->account, 'add_temporary_capability' ] );
		}

	}

	/**
	 * Add integrations with other plugins.
	 *
	 * @since 4.14.2
	 *
	 * @return void
	 */
	public function add_integrations() {

		if ( class_exists( 'WooCommerce' ) && version_compare( WC()->version, '6.3', '>=' ) ) {
			require_once LECO_CLIENT_PORTAL_DIR . 'includes/integrations/class-woocommerce.php';
			require_once LECO_CLIENT_PORTAL_DIR . 'templates/includes/integrations/class-woocommerce.php';
			self::$instance->wc = Integrations\WooCommerce::instance();
			self::$instance->wc->hooks();
			Template_Integrations\WooCommerce::instance()->hooks();
		}

		if ( function_exists( 'gglcptch_plugins_loaded' ) ) {
			require_once LECO_CLIENT_PORTAL_DIR . 'includes/integrations/class-google-captcha.php';
			Integrations\Google_Captcha::instance()->hooks();
		}

	}

	/**
	 * Expose the Notifications object.
	 *
	 * @since 4.15
	 *
	 * @return Notifications
	 */
	public function notifications() {

		return $this->notifications;

	}

	/**
	 * Register custom CMB2 field types.
	 *
	 * @since 4.15.2
	 *
	 * @return void
	 */
	private function register_cmb2_types() {

		add_action( 'cmb2_render_select_multiple', 'leco_cp_cmb2_render_select_multiple_field_type', 10, 5 );
		add_action( 'cmb2_render_multiple_urls', array( 'LECO\Client_Portal\CMB2\Types\Multiple_Urls', 'render' ), 10, 5 );
		add_action( 'cmb2_render_due_date', array( 'LECO\Client_Portal\CMB2\Types\Due_Date', 'render' ), 10, 5 );

		add_filter( 'cmb2_sanitize_select_multiple', 'leco_cp_cmb2_sanitize_select_multiple_callback', 10, 2 );
		add_action( 'cmb2_sanitize_multiple_urls', array( 'LECO\Client_Portal\CMB2\Types\Multiple_Urls', 'sanitize_callback' ), 10, 2 );
		add_action( 'cmb2_sanitize_due_date', array( 'LECO\Client_Portal\CMB2\Types\Due_Date', 'sanitize_callback' ), 10, 4 );

		add_action( 'cmb2_types_esc_multiple_urls', array( 'LECO\Client_Portal\CMB2\Types\Multiple_Urls', 'escape_callback' ), 10, 4 );
		add_action( 'cmb2_types_esc_due_date', array( 'LECO\Client_Portal\CMB2\Types\Due_Date', 'escape_callback' ), 10, 4 );

	}

	/**
	 * Check if the site is set to use Five template (so we will load the Block editor).
	 *
	 * @since 5.0
	 *
	 * @param int|null $post_id The post ID.
	 *
	 * @return bool
	 */
	public function is_five( $post_id = null ) {
		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return false;
		}

		if ( isset( $_GET['post_type'] ) ) {
			if ( 'leco_template' === $_GET['post_type'] || isset( $_GET['page'] ) ) {
				return false;
			}

			if ( 'leco_client' === $_GET['post_type'] && ! $post_id ) {
				$leco_cp_get_template = apply_filters( 'leco_cp_get_template', leco_cp_get_option( 'template', 'five' ) );

				return 'five' === $leco_cp_get_template;
			}
		}

		if ( ! $post_id ) {
			global $wp_query;

			$is_post_editing = isset( $_GET['post'] ) && ! empty( $_GET['post'] ); // phpcs:ignore.

			if ( ! $is_post_editing ) {
				$post_id = $wp_query->get_queried_object_id() ?: ( isset( $_POST['post_ID'] ) ? absint( $_POST['post_ID'] ) : 0 ); // phpcs:ignore.
			} else {
				$post_id = absint( $_GET['post'] ); // phpcs:ignore.
			}
		}

		if ( 'leco_template' === get_post_type( $post_id ) ) {
			return false;
		}

		return ! get_post_meta( $post_id, 'leco_cp_part_0_title', true );
	}

}
