<?php
/**
 * Register the custom REST API routes.
 *
 * @since      5.0
 * @subpackage REST_API/Endpoints
 * @copyright  Copyright (c) 2024, Laura Elizabeth
 * @license    http://opensource.org/licenses/gpl-4.9.php GNU Public License
 * @package    LECO_Client_Portal
 */

namespace LECO\Client_Portal\Rest_Api\Endpoints;

use WP_Error;
use WP_REST_Controller;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Custom REST API routes.
 *
 * @since 5.0
 */
class Controller extends WP_REST_Controller {

	/**
	 * Constructor.
	 *
	 * @since 5.9
	 */
	public function __construct() {
		$this->namespace = 'leco-cp/v1';
		$this->rest_base = 'update-module';

		add_action( 'rest_api_init', array( $this, 'register_routes' ) );
	}

	/**
	 * Register the routes for handling block updates.
	 *
	 * @since 5.0
	 */
	public function register_routes() {
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/', array(
			'methods'             => WP_REST_Server::EDITABLE,
			'callback'            => array( $this, 'update_module_callback' ),
			'permission_callback' => array( $this, 'update_module_permissions_check' ),
		) );
	}

	/**
	 * Permissions check for updating the module.
	 *
	 * @since 5.0
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 */
	public function update_module_permissions_check( WP_REST_Request $request ) {
		if ( ! leco_cp_user_has_access( $request->get_param( 'postID' ) ) ) {
			return new WP_Error( 'rest_forbidden', esc_html__( 'You do not have permissions to update the module.', 'leco-cp' ), array( 'status' => rest_authorization_required_code() ) );
		}

		return true;
	}

	/**
	 * The callback for the route.
	 *
	 * @since 5.0
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function update_module_callback( WP_REST_Request $request ) {
		if ( 'leco_client' !== get_post_type( $request->get_param( 'postID' ) ) ) {
			return new WP_Error( 'rest_invalid_post_type', esc_html__( 'Not a Client Portal portal, bail out.', 'leco-cp' ), array( 'status' => 422 ) );
		}

		switch( $request->get_param( 'action' ) ) {
			case 'mark_as_complete':
			case 'mark_as_active':
			case 'mark_as_inactive':
				$action = str_replace( 'mark_as_', '', $request->get_param( 'action' ) );
				return $this->mark_as( $action, $request );
			default:
				if ( method_exists( $this, $request->get_param( 'action' ) ) ) {
					return $this->{$request->get_param( 'action' )}( $request );
				} else {
					return new WP_Error( 'rest_invalid_action', esc_html__( 'Invalid action.', 'leco-cp' ), array( 'status' => 400 ) );
				}
		}
	}

	/**
	 * Mark the module as complete.
	 *
	 * @since 5.0
	 *
	 * @param string          $action  The action to perform.
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response
	 */
	private function mark_as( string $action, WP_REST_Request $request ): WP_REST_Response {
		$post_id = $request->get_param( 'postID' );
		$key     = $request->get_param( 'key' ); // This is the "phaseID" attribute.
		$content = get_post_field( 'post_content', $post_id );
		$blocks  = parse_blocks( $content );

		$status         = $action === 'complete' ? 'completed' : $action;
		$updated_blocks = $this->update_module_attributes( $blocks, $key, 'status', $status );

		if ( $updated_blocks !== $blocks ) {
			$new_content = serialize_blocks( $updated_blocks );
			wp_update_post( array( 'ID' => $post_id, 'post_content' => $new_content ) );

			/**
			 * Run custom actions when a module has been updated to the status.
			 *
			 * @since 4.15
			 * @since 5.0  Always passing null for the phase.
			 *
			 * @param int        $post_id Project ID.
			 * @param int|null   $phase   Phase.
			 * @param int|string $key     The module ID or the phaseID attribute in Five.
			 */
			if ( $action !== 'inactive' ) {
				$event = 'complete' === $action ? 'completed' : 'activated';
				do_action( "leco_cp_module_$event", $post_id, null, $key );
			}

			return new WP_REST_Response( array( 'message' => esc_html__( 'Module updated successfully.', 'leco-cp' ) ), 200 );
		}

		return new WP_REST_Response( array( 'message' => esc_html__( 'Module did not update.', 'leco-cp' ) ), 200 );
	}

	/**
	 * Update the module attributes.
	 *
	 * @since 5.0
	 *
	 * @param array        $post_blocks The post blocks.
	 * @param string       $key         The module key.
	 * @param string       $attr        The attribute to update.
	 * @param string|array $value       The value to update.
	 * @param bool         $merge_value If we should merge the value into the current value.
	 *
	 * @return array|array[]
	 */
	private function update_module_attributes( array $post_blocks, string $key, string $attr, $value, bool $merge_value = true ): array {
		return array_map( function ( $block ) use ( $key, $attr, $value, $merge_value ) {
			if ( ! isset( $block['innerBlocks'] ) ) {
				return $block;
			}

			if ( ! isset( $block['attrs']['phaseID'] ) || $block['attrs']['phaseID'] !== $key ) {
				$block['innerBlocks'] = $this->update_module_attributes( $block['innerBlocks'], $key, $attr, $value, $merge_value );

				return $block;
			}

			// Don't update if the value is the same.
			if ( isset( $block['attrs'][ $attr ] ) &&  $block['attrs'][ $attr ] === $value ) {
				return $block;
			}

			if ( is_array( $value ) ) {
				if ( ! isset( $block['attrs'][ $attr ] ) ) {
					$block['attrs'][ $attr ] = $value;
				} else {
					$block['attrs'][ $attr ] = $merge_value ? array_merge( (array) $block['attrs'][ $attr ], $value ) : $value;
				}
			} else {
				$block['attrs'][ $attr ] = $value;
			}

			return $block;
		}, $post_blocks );
	}

	/**
	 * Upload client files.
	 *
	 * This method was migrated from LECO_CP_Client_Upload::upload().
	 *
	 * @since 5.0
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	private function upload_client_files( WP_REST_Request $request ) {
		require_once ABSPATH . 'wp-admin/includes/file.php';
		require_once LECO_CLIENT_PORTAL_DIR . 'includes/admin/upload-functions.php';

		$action    = $request->get_param( 'action' );
		$post_id   = $request->get_param( 'postID' );
		$module_id = $request->get_param( 'key' );
		$content   = get_post_field( 'post_content', $post_id );
		$blocks    = parse_blocks( $content );

		header( 'Content-Type: text/html; charset=' . get_option( 'blog_charset' ) );
		send_nosniff_header();
		nocache_headers();

		status_header( 200 );

		// Change upload folder and then move uploaded file.
		leco_cp_create_protection_files( true );
		$_POST['action'] = $action; // This is required for wp_handle_upload() to work properly.
		add_filter( 'upload_dir', 'leco_cp_set_upload_dir' );
		$files   = $request->get_file_params()['files'];
		$results = array();
		$error   = 0;
		foreach ( $files['name'] as $key => $value ) {
			$_file = array(
				'name'     => $value,
				'type'     => $files['type'][ $key ],
				'tmp_name' => $files['tmp_name'][ $key ],
				'error'    => $files['error'][ $key ],
				'size'     => $files['size'][ $key ]
			);

			$file = wp_handle_upload( $_file, array( 'test_form' => true, 'action' => 'upload_client_files' ) );

			// Add a unique ID based on the uploaded timestamp.
			$file['id'] = uniqid( 'leco-', true );

			if ( isset( $file['error'] ) ) {
				$error++;
			}

			$results[ $key ] = $file;
		}
		remove_filter( 'upload_dir', 'leco_cp_set_upload_dir' );

		if ( $error < count( $files ) ) {
			$updated_blocks = $this->update_module_attributes( $blocks, $module_id, 'moduleClientUploads', $results );

			if ( $updated_blocks !== $blocks ) {
				$new_content = serialize_blocks( $updated_blocks );
				wp_update_post( array( 'ID' => $post_id, 'post_content' => $new_content ) );

				do_action( 'leco_cp_client_uploaded', $post_id, null, $module_id, get_current_user_id() );
			}

			return new WP_REST_Response( array( 'data' => $results ), 200 );
		} else {
			return new WP_Error( 'files_not_uploaded', esc_html__( 'Files are not uploaded successfully.', 'leco_cp' ), array( 'status' => 400 ) );
		}
	}

	/**
	 * Delete client files.
	 *
	 * This method was migrated from LECO_CP_Client_Upload::delete().
	 *
	 * @since 5.0
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	private function delete_client_uploads( WP_REST_Request $request ) {
		$post_id   = $request->get_param( 'postID' );
		$module_id = $request->get_param( 'phaseID' );
		$file_name = $request->get_param( 'file' );
		$content   = get_post_field( 'post_content', $post_id );
		$blocks    = parse_blocks( $content );

		$files = $this->get_module_attribute( $content, $module_id, 'moduleClientUploads' );

		// delete file.
		if ( ( $uploads = wp_get_upload_dir() ) && false === $uploads['error'] ) {
			foreach ( $files as $k => $file ) {
				// get file name from the file path.
				$array = explode( '/', $file['file'] );
				$name  = end( $array );
				if ( $name === $file_name ) {
					wp_delete_file( $file['file'] );
					unset( $files[ $k ] );

					break;
				}
			}
		}

		// Reset $files indexes.
		$files = array_values( $files );

		$updated_blocks = $this->update_module_attributes( $blocks, $module_id, 'moduleClientUploads', $files, false );

		if ( $updated_blocks !== $blocks ) {
			$new_content = serialize_blocks( $updated_blocks );
			wp_update_post( array( 'ID' => $post_id, 'post_content' => $new_content ) );

			do_action( 'leco_cp_client_uploaded', $post_id, null, $module_id, get_current_user_id() );
		}

		return new WP_REST_Response( array( 'data' => $files ), 200 );
	}

	/**
	 * Get the module attribute.
	 *
	 * @todo Add support for string values.
	 *
	 * @since 5.0
	 *
	 * @param string $content The post content.
	 * @param string $key     The module key.
	 * @param string $module  The module name.
	 *
	 * @return array|null
	 */
	private function get_module_attribute( $content, $key, $module ) {
		// Pattern to match the specific phase with phaseID and extract moduleClientUploads
		$pattern = '/<!-- wp:leco-cp\/modules {.*?"phaseID":"' . preg_quote( $key, '/' ) . '".*?"' . $module . '":(.*?)} \/-->/';

		// Search for the pattern in the content
		if ( preg_match( $pattern, $content, $matches ) ) {
			// Decode the JSON string to an array
			$value = json_decode( $matches[1], true );

			if ( json_last_error() === JSON_ERROR_NONE ) {
				return $value;
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

}
