import { __ } from '@wordpress/i18n';
import apiFetch from '@wordpress/api-fetch';
import { FormFileUpload, DropZone, Spinner } from '@wordpress/components';
import { useState, useEffect } from '@wordpress/element';
import Icons from '../components/icons';

import UploadsModal from '../components/UploadsModal';

const ClientUploadsModule = ( props ) => {
	const { phaseID, postID, files } = props;
	const { lecoCPVars } = window;
	const { max_upload_size } = lecoCPVars;

	const [ attributes, setAttributes ] = useState( {
		phaseID: 0,
		postID: 0,
	} );

	const clientFiles = files ? JSON.parse( files ) : [];

	const [ uploadedFiles, setUploadedFiles ] = useState( clientFiles );

	const [ uploadingFiles, setUploadingFiles ] = useState( [] );

	const [ loading, setLoading ] = useState( false );

	const [ modelOpen, setModelOpen ] = useState( false );

	useEffect( () => {
		setAttributes( {
			phaseID,
			postID,
		} );
	}, [ phaseID, postID ] );

	const saveUpdates = ( newFiles ) => {
		setUploadedFiles( [ ...uploadedFiles, ...newFiles ] );
		setUploadingFiles( [] );

		setLoading( false );
	};

	const deleteFile = async ( file ) => {
		setLoading( true );

		const name = file.file.split( '/' ).pop();
		const id = file?.id;

		const parentItem = document.getElementById( id );

		parentItem.classList.add( 'deleting' );

		await apiFetch( {
			path: `leco-cp/v1/update-module/`,
			method: 'POST',
			data: {
				...attributes,
				file: name,
				action: 'delete_client_uploads',
			},
		} )
			.then( ( success ) => {
				const v = [ ...success.data ];
				setUploadedFiles( v );
				parentItem.classList.remove( 'deleting' );
			} )
			.catch( ( error ) => {
				console.log( error );
				parentItem.classList.remove( 'deleting' );
			} );

		setLoading( false );
	};

	const getFilesFromButton = ( event ) => {
		uploadFiles( event.currentTarget.files );
	};

	const basename = ( path ) => {
		return path.split( '/' ).reverse()[ 0 ];
	};

	const uploadFiles = async ( files ) => {
		if ( ! files.length ) {
			return;
		}

		setLoading( true );
		setUploadingFiles( files );

		const data = new FormData();

		data.append( 'action', 'upload_client_files' );
		data.append( 'postID', postID );
		data.append( 'key', phaseID );
		for ( let i = 0; i < files.length; i++ ) {
			data.append( 'files[]', files[ i ] );
		}

		await apiFetch( {
			path: `leco-cp/v1/update-module/`,
			method: 'POST',
			body: data,
		} )
			.then( ( result ) => {
				saveUpdates( result.data );
			} )
			.catch( ( error ) => {
				console( error );
			} );
	};

	return (
		<div className="leco-cp__file-upload">
			<FormFileUpload
				onChange={ getFilesFromButton }
				className="choose-files"
				disabled={ loading }
				multiple={ true }
			>
				<DropZone onFilesDrop={ uploadFiles } />
				<span>
					{ __( 'Click or drop files here to add more', 'leco-cp' ) }
					{ loading && <Spinner /> }
				</span>
				<p className="max-upload-size">
					{ __( 'Maximum upload file size:', 'leco-cp' ) }{ ' ' }
					{ max_upload_size ?? '0MB' }
				</p>
			</FormFileUpload>

			{ ( uploadedFiles || uploadingFiles ) && (
				<ul className="files">
					{ uploadedFiles.map( ( file, id ) => {
						const fileURL = window.location.href + // eslint-disable-line
							'file/' +
							btoa( phaseID + '|' + file.id ) +
							'/';

						return (
							<li id={ file?.id } key={ id }>
								<a href={ fileURL } download>
									{ basename( file.url ) }
									<span className="icon-download">
										<Icons.download />
									</span>
								</a>
								<button
									onClick={ () => {
										setModelOpen( file.id );
									} }
									className="file-delete"
									title={ __( 'Delete', 'leco-cp' ) }
								>
									<span className="icon-file-delete">
										<Icons.delete />
									</span>
								</button>

								{ modelOpen === file.id && (
									<UploadsModal
										setModelOpen={ setModelOpen }
										deleteFile={ deleteFile }
										file={ file }
									/>
								) }
							</li>
						);
					} ) }
					{ uploadingFiles &&
						[ ...uploadingFiles ].map( ( file, id ) => {
							return (
								<li key={ id }>
									<span>{ file.name.toLowerCase() }</span>{ ' ' }
									<Spinner />
								</li>
							);
						} ) }
				</ul>
			) }
		</div>
	);
};

export default ClientUploadsModule;
