import { __ } from '@wordpress/i18n';
import { Button, Popover, ColorPicker, PanelRow } from '@wordpress/components';
import { useState } from '@wordpress/element';

const CPColorPicker = ( { title, meta, setMeta, metaKey, colorSlug } ) => {
	const [ isVisible, setIsVisible ] = useState( false );
	const [ color, setColor ] = useState( meta[ metaKey ] );

	/*
	 * Update the color variable with the new color.
	 *
	 * @param {string} newColor The new color value.
	 *
	 * @return {void}
	 */
	const updateEditorColor = ( newColor ) => {
		const stylesWrapper = document.querySelector( '.editor-styles-wrapper' );

		stylesWrapper.style.setProperty( `--wp--preset--color--${ colorSlug }`, newColor );
	};

	/*
	 * Update state and editor color with the new color.
	 *
	 * @param {string} newColor The new color value.
	 * @return {void}
	 */
	const colorPicker = ( newColor ) => {
		setColor( newColor );

		updateEditorColor( newColor );
	};

	/*
	 * Toggle the color picker visibility.
	 *
	 * @return {void}
	 */
	const toggleVisible = () => {
		setIsVisible( ( state ) => ! state );

		if ( isVisible ) {
			setMeta( {
				...meta,
				[ metaKey ]: color,
			} );
		}
	};

	/*
	 * Return the color to the original value.
	 *
	 * @return {void}
	 */
	const cancelColorChange = () => {
		setIsVisible( ( state ) => ! state );
		setColor( meta[ metaKey ] );
		updateEditorColor( meta[ metaKey ] );
	};

	/*
	 * Convert hex to rgb.
	 *
	 * @param {string} hex The hex color value.
	 *
	 * @return {string} rgb The rgb color value.
	 */
	const hexToRgb = ( hex ) =>
		hex.replace( /^#?([a-f\d])([a-f\d])([a-f\d])$/i
			, ( m, r, g, b ) => '#' + r + r + g + g + b + b )
			.substring( 1 ).match( /.{2}/g )
			.map( ( x ) => parseInt( x, 16 ) );

	/*
	 * Adds the background and text color to the button.
	 * Making sure the text color is readable.
	 *
	 * @return {string} inline html styles.
	 */
	const setColors = () => {
		const rgbVal = hexToRgb( color );

		const brightness = Math.round( ( ( parseInt( rgbVal[ 0 ] ) * 299 ) + ( parseInt( rgbVal[ 1 ] ) * 587 ) + ( parseInt( rgbVal[ 2 ] ) * 114 ) ) / 1000 );

		const textColour = ( brightness > 125 ) ? '#000000' : '#ffffff';

		return {
			background: color,
			color: textColour,
		};
	};

	return (
		<>
			<h2 className="leco-cp-panel-header">
				{ title }
			</h2>
			<PanelRow>
				<Button
					className="components-color-palette__custom-color-button"
					onClick={ toggleVisible }
					style={ setColors() }
				>
					{ isVisible ? __( 'Apply this color', 'leco-cp' ) : __( 'Select color', 'leco-cp' ) }
				</Button>
				{ isVisible &&
					<>
						<Button
							onClick={ cancelColorChange }
							variant="tertiary"
						>
							{ __( 'Cancel', 'leco-cp' ) }
						</Button>
						<Popover>
							<ColorPicker
								color={ color }
								onChange={ colorPicker }
								enableAlpha
								defaultValue={ meta[ metaKey ] }
							/>
						</Popover>
					</>
				}
			</PanelRow>
		</>
	);
};

export default CPColorPicker;
