import { __ } from '@wordpress/i18n';
import { registerPlugin } from '@wordpress/plugins';
import { PluginDocumentSettingPanel } from '@wordpress/edit-post';
import { more } from '@wordpress/icons';
import { ToggleControl, SelectControl } from '@wordpress/components';
import { dispatch } from '@wordpress/data';
import { useEntityProp } from '@wordpress/core-data';

import CPColorPicker from './components/color-picker';
import ClientPicker from './components/client-picker';
import CPSettings from './components/cp-settings';

const SideBar = () => {
	const [ meta, setMeta ] = useEntityProp( 'postType', 'leco_client', 'meta' );
	const prefix = 'leco_cp_5_';

	return (
		<PluginDocumentSettingPanel
			title={ __( 'Client Portal Settings', 'leco-cp' ) }
			initialOpen="true"
		>
			<CPSettings
				title={ __( 'Use default settings for empty fields', 'leco-cp' ) }
				description={ __( 'If you select “Use default settings”, we’ll load the values from Client Portal Settings into the empty fields in your portal. If you select “Don\'t use empty fields”, we will leave all empty fields blank.', 'leco-cp' ) }
			>
				<ToggleControl
					label={
						meta[ `${ prefix }fallback_values` ]
							? __( 'Use default settings', 'leco-cp' )
							: __( 'Don\'t use empty fields.', 'leco-cp' )
					}
					checked={ meta[ `${ prefix }fallback_values` ] ?? true }
					onChange={ ( value ) => {
						setMeta( {
							...meta,
							[ `${ prefix }fallback_values` ]: value,
						} );
					} }
				/>
			</CPSettings>
			<hr />
			<CPSettings
				title={ __( 'Show mark complete ticks?', 'leco-cp' ) }
				description={ __( 'This will enable the client side "Mark as Complete" feature for each module. The default is "Show for modules with a due date".', 'leco-cp' ) }
			>
				<SelectControl
					value={ meta[ `${ prefix }show_mark_as_complete` ] ?? 'show_with_due_date' }
					options={ [
						{ label: __( 'Show for modules with a due date', 'leco-cp' ), value: 'show_with_due_date' },
						{ label: __( 'Show for all modules', 'leco-cp' ), value: 'show' },
						{ label: __( 'No', 'leco-cp' ), value: 'hide' },
					] }
					onChange={ ( value ) => {
						setMeta( {
							...meta,
							[ `${ prefix }show_mark_as_complete` ]: value,
						} );
					} }
				/>
			</CPSettings>
			<hr />
			<CPSettings
				title={ __( 'Public Portal', 'leco-cp' ) }
				description={ __( 'Set this to "Yes" for public portals. If set to "Yes", anyone can view this portal as long as they have the URL.', 'leco-cp' ) }
			>
				<ToggleControl
					label={ meta[ `${ prefix }public_portal` ]
						? 'Yes'
						: 'No'
					}
					checked={ meta[ `${ prefix }public_portal` ] ?? false }
					onChange={ ( value ) => {
						setMeta( {
							...meta,
							[ `${ prefix }public_portal` ]: value,
						} );
					} }
				/>
			</CPSettings>
			{
				! meta[ `${ prefix }public_portal` ] && (
					<>
						<hr />
						<CPSettings
							title={ __( 'Client Accounts', 'leco-cp' ) }
							description={ __( 'Only the selected client account can view this page on front end. If you set a post password to protect it, this option will be omitted.', 'leco-cp' ) }
						>
							<ClientPicker
								meta={ meta }
								setMeta={ setMeta }
							/>
						</CPSettings>
					</>
				)
			}
		</PluginDocumentSettingPanel>
	);
};

const ColorPanel = () => {
	const [ meta, setMeta ] = useEntityProp( 'postType', 'leco_client', 'meta' );
	const prefix = 'leco_cp_5_';

	return (
		<PluginDocumentSettingPanel
			title={ __( 'Client Portal Colors', 'leco-cp' ) }
			initialOpen="false"
		>
			<CPColorPicker
				title={ __( 'Primary Background Color', 'leco-cp' ) }
				meta={ meta }
				setMeta={ setMeta }
				metaKey={ `${ prefix }primary_color` }
				colorSlug="leco-cp-primary"
			/>
			<hr />
			<CPColorPicker
				title={ __( 'Secondary Background Color', 'leco-cp' ) }
				meta={ meta }
				setMeta={ setMeta }
				metaKey={ `${ prefix }secondary_color` }
				colorSlug="leco-cp-secondary"
			/>
			<hr />
			<CPColorPicker
				title={ __( 'Tertiary Background Color', 'leco-cp' ) }
				meta={ meta }
				setMeta={ setMeta }
				metaKey={ `${ prefix }tertiary_color` }
				colorSlug="leco-cp-tertiary"
			/>
			<hr />
			<CPColorPicker
				title={ __( 'Primary Text Color', 'leco-cp' ) }
				meta={ meta }
				setMeta={ setMeta }
				metaKey={ `${ prefix }primary_text_color` }
				colorSlug="leco-cp-primary-text"
			/>
			<hr />
			<CPColorPicker
				title={ __( 'Secondary Text Color', 'leco-cp' ) }
				meta={ meta }
				setMeta={ setMeta }
				metaKey={ `${ prefix }secondary_text_color` }
				colorSlug="leco-cp-secondary-text"
			/>
			<hr />
			<CPColorPicker
				title={ __( 'Tertiary Text Color', 'leco-cp' ) }
				meta={ meta }
				setMeta={ setMeta }
				metaKey={ `${ prefix }tertiary_text_color` }
				colorSlug="leco-cp-tertiary-text"
			/>
			<hr />
		</PluginDocumentSettingPanel>
	);
};

registerPlugin( 'leco-cp-sidebar', {
	icon: more,
	render: SideBar,
} );

registerPlugin( 'leco-cp-colors', {
	icon: more,
	render: ColorPanel,
} );

// Add users endpoint.
dispatch( 'core' ).addEntities( [
	{
		name: 'user',
		kind: 'root',
		baseURL: '/wp/v2/users',
	},
] );
